export function joinURL(base, pathname) {
    pathname = pathname
        .split('/')
        .filter((v) => v.length > 0)
        .join('/');
    if (!base.endsWith('/') && pathname.length > 0)
        base += '/';
    return base + pathname;
}
/**
 * @param url - URL (can be relative)
 * @param base - the base URL (must be absolute)
 */
export function withBase(url, base) {
    if (!url.startsWith('https://') && !url.startsWith('http://')) {
        return joinURL(base, url);
    }
    return url;
}
export function resolveServerUrl(template, variables) {
    for (const [key, value] of Object.entries(variables)) {
        template = template.replaceAll(`{${key}}`, value);
    }
    return template;
}
export function resolveRequestData(pathname, { path, query }) {
    for (const key in path) {
        if (path[key] === '')
            continue;
        if (Array.isArray(path[key])) {
            pathname = pathname.replace(`{${key}}`, path[key].join('/'));
        }
        else {
            pathname = pathname.replace(`{${key}}`, String(path[key]));
        }
    }
    const searchParams = new URLSearchParams();
    for (const key in query) {
        if (Array.isArray(query[key])) {
            for (const value of query[key]) {
                if (!query[key])
                    continue;
                searchParams.append(key, String(value));
            }
        }
        else if (query[key] && query[key] !== '') {
            searchParams.set(key, String(query[key]));
        }
    }
    return searchParams.size > 0 ? `${pathname}?${searchParams}` : pathname;
}
